/*
 * File: fileio.c
 * Date: 2011-11-01
 *
 * Evsiz (evsiz@this-voice.org)
 * Rho released under the Creative Commons NonCommercial ShareAlike License 3.0
 * See http://creativecommons.org for details
 *
 * Rho is a fork of the Linux port of MicroEMACS developed by Humberto
 * Ortiz-Zuazaga and Jean-Pierre Demailly. Their work forked the MicroEMACS
 * of Daniel M. Lawrence which was in turn a fork of uemacs by Dave G.
 * Conroy. MicroEmacs was copyrighted 1993 by Mr. Lawrence with a
 * stipulation that it could not be used commercially. Uemacs was released
 * by Mr. Conroy into the public domain in 1985.
 *
 */


/***

The routines in this file read and write files from the disk. All of
the knowledge about fileio is here.

***/

#include    <stdio.h>
#include    <stdlib.h>
#include    <string.h>
#include    <sys/stat.h>
#include    "rho.h"
#include    "fileio.h"
#include    "file.h"
#include    "messages.h"



FILE *ffp;                      /* File pointer, all functions. */
static int eofflag;             /* end-of-file flag */

/*++

Open a file for reading.

++*/
int ffropen(fn)
char *fn;
{
    if ((ffp = fopen(fn, "r")) == NULL)
        return (FIOFNF);

    eofflag = FALSE;
    return (FIOSUC);
}



/*++

Open a file for writing.

++*/
int ffwopen(fn, mode)
char *fn;
char *mode;                     /* mode to open file for */
{
    char xmode[6];              /* extended file open mode */

    /* nonstandard line terminators? */
    if (*lterm) {
        /* open in binary mode */
        strcpy(xmode, mode);
        strcat(xmode, "b");
        ffp = fopen(fn, xmode);
    } else {
        /* open in ascii(text) mode */
        ffp = fopen(fn, mode);
    }

    if (ffp == NULL) {
        mlwrite(TEXT155);       /*"Cannot open file for writing" */
        return (FIOERR);
    }

    return (FIOSUC);
}



/*++

Close a file. 

++*/
int ffclose()
{
    /* free this since we do not need it anymore */
    if (fline) {
        free(fline);
        fline = NULL;
    }

    if (fclose(ffp) != FALSE) {
        mlwrite(TEXT156);       /* "Error closing file" */
        return (FIOERR);
    }
    return (FIOSUC);
}

/*++

Write a line to the already opened file. The "buf" points to the buffer,
and the "nbuf" is its length, less the free newline. Return the status.
Check only at the newline.

++*/
int ffputline(buf, nbuf)
char buf[];
int nbuf;

{
    int i;                      /* index into line to write */
    char *lptr;                 /* ptr into the line terminator */
    for (i = 0; i < nbuf; ++i)
        putc(buf[i], ffp);

    /* write out the appropriate line terminator(s) */
    if (*lterm) {
        lptr = &lterm[0];
        while (*lptr)
            putc(*lptr++, ffp);
    } else {
        putc('\n', ffp);
    }

    /* check for write errors */
    if (ferror(ffp)) {
        mlwrite(TEXT157);       /* "Write I/O error" */
        return (FIOERR);
    }

    return (FIOSUC);
}



/*++

Read a line from a file, and store the bytes in the supplied buffer. The
"nbuf" is the length of the buffer. Complain about long lines and lines
at the end of the file that don't have a newline present. Line length
restricted to NSTRING.  Check for I/O errors too. Return status.

++*/
int ffgetline(nbytes)
int *nbytes;

{
    int c;                      /* current character read */
    int i;                      /* current index into fline */

    /* if we are at the end...return it */
    if (eofflag)
        return (FIOEOF);

    /* dump fline if it ended up too big */
    if (flen > NSTRING && fline != NULL) {
        free(fline);
        fline = NULL;
    }

    /* if we don't have an fline, allocate one */
    if (fline == NULL)
        fline = malloc(flen = (size_t) NSTRING);


    /* read the line in */
    i = 0;
    while ((c = getc(ffp)) != EOF && c != '\n') {
        fline[i++] = c;
        /* if it's longer, get more room */
        if (i >= (int) flen) {
            flen *= 2;
            if ((fline = realloc(fline, flen)) == NULL) {
                return (FIOMEM);
            }
        }
    }

    /* dump any extra line terminators at the end */
    while (i > 0 && (fline[i - 1] == 10 || fline[i - 1] == 13))
        i--;

    /* and save the length for our caller... */
    *nbytes = i;


    /* test for any errors that may have occured */
    if (c == EOF) {
        if (ferror(ffp)) {
            mlwrite(TEXT158);   /* "File read error" */
            return (FIOERR);
        }

        if (i != 0)
            eofflag = TRUE;
        else
            return (FIOEOF);
    }

    /* terminate string */
    fline[i] = 0;
    return (FIOSUC);
}


/*++

Check for existence of fname on disk.

++*/
int fexist(fname)
char *fname;                    /* filename to check */

{
    struct stat st;

    (void) stat(fname, &st);
    if (S_ISREG(st.st_mode))
        return (TRUE);

    return (FALSE);
}
